package additions

import (
	"fmt"
	"unsafe"

	log "github.com/sirupsen/logrus"
	"golang.org/x/sys/unix"
)

type system struct {
	pageSize uint64
}

// NewSystem() Initalizes OS specific interfaces
// FreeBSD:
//   - nothing yet
func NewSystem() (System, error) {

	// determine page size for memory calculations
	pageSz, err := SysctlUint("vm.stats.vm.v_page_size")
	if err != nil {
		return nil, err
	}

	return &system{
		pageSize: pageSz,
	}, nil
}

func (sys *system) SetHostname(hostname *string) error {
	return nil
}

func (sys *system) GetFreeMemory() uint64 {
	free, err := SysctlUint("vm.stats.vm.v_free_count")
	if err != nil {
		log.Error("GetFreeMemory(): ", err)
		return 0
	}
	free *= sys.pageSize

	return free

}

func (sys *system) GetTotalMemory() uint64 {
	total, err := SysctlUint("hw.physmem")
	if err != nil {
		log.Error("GetTotalMemory(): ", err)
		return 0
	}

	return total
}

func (sys *system) GetUsedMemory() uint64 {
	return (sys.GetTotalMemory() - sys.GetFreeMemory())
}

// TODO: replace by real motd and not just this text.
func (sys *system) GetMotd() (string, error) {
	return "generic gnmi-target motd. Not real OS motd, sorry.", nil
}

func (sys *system) SetMotd(message string) error {
	return nil
}

/*
 *  This func copied from gopsutils
 * https://github.com/shirou/gopsutil/blob/master/internal/common/common_freebsd.go
 */
func SysctlUint(mib string) (uint64, error) {
	buf, err := unix.SysctlRaw(mib)
	if err != nil {
		return 0, err
	}
	if len(buf) == 8 { // 64 bit
		return *(*uint64)(unsafe.Pointer(&buf[0])), nil
	}
	if len(buf) == 4 { // 32bit
		t := *(*uint32)(unsafe.Pointer(&buf[0]))
		return uint64(t), nil
	}
	return 0, fmt.Errorf("unexpected size: %s, %d", mib, len(buf))
}

func (sys *system) GetSoftwareVersion() (string, error) {
	return "no such information yet.", nil
}
